from django.db import transaction
from django.views.decorators.http import require_http_methods
from rest_framework.views import APIView

from xj_user.utils.user_wrapper import user_authentication_wrapper
from ..service.subitem_service import SubitemService
from ..utils.custom_response import util_response
from ..utils.custom_tool import parse_data


class SubitemApis(APIView):

    @require_http_methods(['GET'])
    @user_authentication_wrapper
    def list(self, *args, user_info=None, **kwargs):
        request_params = parse_data(self)
        request_params["user_id"] = user_info.get("user_id", None)
        data, err = SubitemService.list(params=request_params)

        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @require_http_methods(['GET'])
    def detail(self, *args, **kwargs, ):
        request_params = parse_data(self)
        pk = kwargs.get("pk") or request_params.get("id") or request_params.get("subitem_id")
        if not pk:
            return util_response(err=1000, msg="参数错误")
        data, err = SubitemService.detail(pk=pk)
        if err:
            return util_response(err=1001, msg=err)
        return util_response(data=data)

    @require_http_methods(['POST'])
    def add(self, *args, **kwargs, ):
        params = parse_data(self)
        data, err = SubitemService.add(params)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @require_http_methods(['POST'])
    def batch_add(self, *args, **kwargs, ):
        """批量添加报名分项"""
        params_list = parse_data(self)
        if not isinstance(params_list, list) or not params_list:
            return util_response(err=1000, msg="应该传[{},{},....]")

        sid = transaction.savepoint()
        try:
            for params in params_list:
                enroll_id = params.get("enroll_id", None)
                if not enroll_id:
                    transaction.savepoint_rollback(sid)
                    return util_response(err=1001, msg="enroll_id 必传")
                is_can_add = SubitemService.check_num(enroll_id)
                if not is_can_add:
                    transaction.savepoint_rollback(sid)
                    return util_response(err=1002, msg="超出需求份数，或者enroll_id 不存在。")
                data, err = SubitemService.add(params)
                if err:
                    transaction.savepoint_rollback(sid)
                    return util_response(err=1000, msg=err)
        except Exception as e:
            transaction.savepoint_rollback(sid)
            return util_response(err=1000, msg=str(e))

        # 清除mysql事务点
        transaction.clean_savepoints()
        return util_response()

    @require_http_methods(['PUT'])
    def batch_edit(self, *args, **kwargs, ):
        """批量添加报名分项"""
        params_list = parse_data(self)
        if not isinstance(params_list, list) or not params_list:
            return util_response(err=1000, msg="应该传[{},{},....]")

        sid = transaction.savepoint()
        try:
            for params in params_list:
                enroll_id = params.get("enroll_id", None)
                if not enroll_id:
                    transaction.savepoint_rollback(sid)
                    return util_response(err=1001, msg="enroll_id 必传")
                data, err = SubitemService.edit(params)
                if err:
                    transaction.savepoint_rollback(sid)
                    return util_response(err=1000, msg=err)
        except Exception as e:
            transaction.savepoint_rollback(sid)
            return util_response(err=1000, msg=str(e))

        # 清除mysql事务点
        transaction.clean_savepoints()
        return util_response()

    @require_http_methods(['PUT'])
    def edit(self, *args, **kwargs, ):
        params = parse_data(self)
        subitem_id = params.pop("id", None) or kwargs.pop("pk", None)
        data, err = SubitemService.edit(params, subitem_id)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @require_http_methods(['PUT'])
    def edit_by_enroll_id(self, *args, **kwargs, ):
        params = parse_data(self)
        enroll_id = params.pop("id", None) or kwargs.pop("enroll_id", None)
        data, err = SubitemService.batch_edit(params, enroll_id)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    # @require_http_methods(['DELETE'])
    # def delete(self, *args, **kwargs, ):
    #     params = parse_data(self)
    #     subitem_id = params.pop("id", None) or kwargs.pop("pk", None)
    #     data, err = SubitemService.delete(params, subitem_id)
    #     if err:
    #         return util_response(err=1000, msg=err)
    #     return util_response(data=data)

    @require_http_methods(['GET'])
    def extend_field(self, *args, **kwargs, ):
        request_params = parse_data(self)
        is_pagination = request_params.pop("is_pagination", 1)
        is_pagination = int(is_pagination)
        data, err = SubitemService.extend_field(params=request_params, is_pagination=is_pagination)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)



from django.contrib import admin

from .models import *


class EnrollAdmin(admin.ModelAdmin):
    list_display = (
        'id', 'thread_id', 'trading_relate', 'region_code', 'occupy_room', 'enroll_status_code', 'min_number',
        'max_number', 'min_count_apiece', 'max_count_apiece', 'enroll_rule_group', 'price', 'count', 'unit', 'fee',
        'reduction', 'subitems_amount', 'amount', 'paid_amount', 'unpaid_amount', 'commision', 'deposit', 'hide_price',
        'hide_user', 'has_repeat', 'has_subitem', 'has_audit', 'need_vouch', 'need_deposit',
        'need_imprest', 'enable_pool', 'pool_limit', 'pool_stopwatch', 'open_time', 'close_time', 'launch_time',
        'finish_time', 'spend_time', 'create_time', 'update_time', 'snapshot', 'remark',
    )
    search_fields = ('id', 'thread_id', 'trading_relate', 'region_code', 'occupy_room', 'enroll_status_code', 'min_number',)
    fields = (
        'id', 'thread_id', 'trading_relate', 'region_code', 'occupy_room', 'enroll_status_code', 'min_number', 'max_number', 'min_count_apiece', 'max_count_apiece', 'enroll_rule_group', 'price',
        'count', 'unit', 'fee', 'has_repeat', 'has_subitem', 'has_audit', 'need_vouch', 'need_deposit', 'need_imprest', 'enable_pool', 'pool_limit', 'pool_stopwatch', 'open_time', 'close_time',
        'launch_time', 'finish_time', 'spend_time', 'create_time', 'update_time', 'snapshot', 'remark',
    )
    readonly_fields = ['id', 'update_time']


class EnrollSubitemAdmin(admin.ModelAdmin):
    list_display = ('id', 'enroll_id', 'name', 'price', 'count', 'unit', 'amount', 'description', 'remark', 'enroll_subitem_status_code',
                    'field_1', 'field_2', 'field_3', 'field_4', 'field_5', 'field_6', 'field_7', 'field_8', 'field_9',
                    'field_10',)
    search_fields = ('id', 'enroll_id', 'name', 'price', 'count', 'unit', 'amount', 'description', 'remark',
                     'field_1', 'field_2', 'field_3', 'field_4', 'field_5', 'field_6', 'field_7', 'field_8', 'field_9',
                     'field_10',)
    fields = ('id', 'enroll_id', 'name', 'price', 'count', 'unit', 'amount', 'description', 'remark', 'enroll_subitem_status_code',
              'field_1', 'field_2', 'field_3', 'field_4', 'field_5', 'field_6', 'field_7', 'field_8', 'field_9',
              'field_10',)
    readonly_fields = ['id']


class EnrollAuthStatusAdmin(admin.ModelAdmin):
    list_display = ('id', 'value')
    search_fields = ('id', 'value')
    fields = ('id', 'value',)
    readonly_fields = ['id']


class EnrollPayStatusAdmin(admin.ModelAdmin):
    list_display = ('id', 'value')
    search_fields = ('id', 'value')
    fields = ('id', 'value',)
    readonly_fields = ['id']


class EnrollRecordAdmin(admin.ModelAdmin):
    list_display = ('id', 'enroll', 'user_id', 'enroll_auth_status_id', 'enroll_pay_status_id', 'enroll_status_code',
                    'create_time', 'price', 'deposit', 'count', 'main_amount', 'coupon_amount', 'again_reduction',
                    'subitems_amount', 'deposit_amount', 'amount', 'paid_amount', 'unpaid_amount',
                    'fee', 'photos', 'files', 'score', 'reply', 'remark',)
    search_fields = ('id', 'enroll', 'user_id', 'enroll_auth_status_id', 'enroll_pay_status_id', 'enroll_status_code',
                     'create_time', 'price', 'deposit', 'count', 'main_amount', 'coupon_amount', 'again_reduction',
                     'subitems_amount', 'deposit_amount', 'amount', 'paid_amount', 'unpaid_amount',
                     'fee', 'photos', 'files', 'score', 'reply', 'remark',)
    fields = ('id', 'enroll', 'user_id', 'enroll_auth_status_id', 'enroll_pay_status_id', 'enroll_status_code',
              'create_time', 'price', 'deposit', 'count', 'main_amount', 'coupon_amount', 'again_reduction',
              'subitems_amount', 'deposit_amount', 'amount', 'paid_amount', 'unpaid_amount',
              'fee', 'photos', 'files', 'score', 'reply', 'remark',)
    readonly_fields = ['id']


class EnrollRecordSubitemAdmin(admin.ModelAdmin):
    list_display = ('id', 'enroll_record', 'enroll_subitem', 'user_id', 'price', 'count', 'subitem_amount', 'enroll_subitem_status_code', "reply", "remark", "files", "photos",)
    search_fields = ('id', 'enroll_record', 'enroll_subitem', 'user_id', 'price', 'count', 'subitem_amount',)
    fields = ('id', 'enroll_record', 'enroll_subitem', 'user_id', 'price', 'count', 'subitem_amount', 'enroll_subitem_status_code', "reply", "remark", "files", "photos",)
    readonly_fields = ['id', ]


class EnrollRuleGroupAdmin(admin.ModelAdmin):
    list_display = ('id', "classifyt_id", 'rule_group', 'category_id', 'description',)
    search_fields = ('id', "classifyt_id", 'rule_group', 'category_id', 'description',)
    fields = ('id', 'rule_group', "classifyt_id", 'category_id', 'description',)
    readonly_fields = ['id', ]


class EnrollRuleValuateAdmin(admin.ModelAdmin):
    list_display = ('id', 'enroll_rule_group', 'name', 'type', 'field', 'expression_string', 'sort')
    search_fields = ('id', 'enroll_rule_group', 'name', 'type', 'field', 'expression_string',)
    fields = ('id', 'enroll_rule_group', 'name', 'type', 'field', 'expression_string', 'sort')
    readonly_fields = ['id', ]
    ordering = ("-enroll_rule_group", '-sort',)


class EnrollStatusCodeAdmin(admin.ModelAdmin):
    list_display = ('id', 'enroll_status_code', 'value', "config")
    search_fields = ('id', 'enroll_status_code', 'value')
    fields = ('id', 'enroll_status_code', 'value', "config")
    readonly_fields = ['id', ]


admin.site.register(EnrollStatusCode, EnrollStatusCodeAdmin)
admin.site.register(Enroll, EnrollAdmin)
admin.site.register(EnrollAuthStatus, EnrollAuthStatusAdmin)
admin.site.register(EnrollPayStatus, EnrollPayStatusAdmin)
admin.site.register(EnrollRecord, EnrollRecordAdmin)
admin.site.register(EnrollSubitem, EnrollSubitemAdmin)
admin.site.register(EnrollSubitemRecord, EnrollRecordSubitemAdmin)
admin.site.register(EnrollRuleGroup, EnrollRuleGroupAdmin)
admin.site.register(EnrollRuleValuate, EnrollRuleValuateAdmin)



# encoding: utf-8
"""
@project: djangoModel->enroll_record_serivce
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 用户报名记录
@created_time: 2022/9/17 15:45
"""
from django.core.paginator import Paginator

from ..models import EnrollRecord, Enroll
from ..utils.custom_tool import format_params_handle


class EnrollRecordServices:
    @staticmethod
    def check_can_add(enroll_id, user_id):
        enroll_obj = Enroll.objects.filter(id=enroll_id).first()
        if not enroll_obj:
            # 传递的报名ID不正确
            return False
        need_num = enroll_obj.to_json().get("count")

        enroll_record_obj = EnrollRecord.objects.filter(enroll_id=enroll_id).exclude(enroll_status_code=124)
        if need_num <= enroll_record_obj.count():
            # 报名数量达到限制
            return False

        this_user_record_obj = enroll_record_obj.filter(user_id=user_id).first()
        if this_user_record_obj:
            # 当前用户报名过了，不允许报名了
            return False

        return True

    @staticmethod
    def record_add(params):
        params = format_params_handle(
            param_dict=params,
            filter_filed_list=[
                "enroll_id",
                "user_id",
                "enroll_auth_status_id",
                "enroll_pay_status_id",
                "enroll_status_code",
                "create_time",
                "price",
                "deposit",
                "count",
                "main_amount",
                "coupon_amount",
                "again_reduction",
                "subitems_amount",
                "deposit_amount",
                "amount",
                "paid_amount",
                "unpaid_amount",
                "fee",
                "photos",
                "files",
                "score",
                "reply",
                "remark",
            ],
        )
        # print("params", params)
        enroll_id = params.get("enroll_id")
        if not enroll_id:
            return None, "enroll_id不能为空"
        try:
            # instance, is_create = EnrollRecord.objects.get_or_create(user_id=params["user_id"], enroll_id=params["enroll_id"], defaults=params)
            instance = EnrollRecord.objects.create(**params)
        except Exception as e:
            return None, str(e)
        return instance.to_json(), None

    @staticmethod
    def record_list(params, need_pagination=None):
        size = params.pop('size', 10)
        page = params.pop('page', 1)
        params = format_params_handle(
            param_dict=params,
            filter_filed_list=[
                "id",
                "enroll_id",
                "user_id",
                "enroll_auth_status_id",
                "enroll_pay_status_id",
                "enroll_status_code",
                "create_time",
                "price",
                "deposit",
                "count",
                "main_amount",
                "coupon_amount",
                "again_reduction",
                "subitems_amount",
                "deposit_amount",
                "amount",
                "paid_amount",
                "unpaid_amount",
                "fee",
                "photos",
                "files",
                "score",
                "reply",
                "remark",
            ],
        )
        # PS 124状态码任务取消报名，不予显示
        enroll_obj = EnrollRecord.objects.filter(**params).exclude(enroll_status_code=124).values()
        if not need_pagination:
            return list(enroll_obj), None
        count = enroll_obj.count()
        paginator = Paginator(enroll_obj, size)
        enroll_obj = paginator.page(page)
        enroll_list = list(enroll_obj.object_list)
        data = {'total': count, "page": page, "size": size, 'list': enroll_list}
        return data, None

    @staticmethod
    def record_edit(params, pk):
        pk = params.pop("id", None) or pk
        # print("pk", pk)
        params = format_params_handle(
            param_dict=params,
            filter_filed_list=[
                "enroll", "user_id", "enroll_auth_status_id", "enroll_pay_status_id", "enroll_status_code", "create_time", "price", "deposit", "count", "main_amount", "coupon_amount",
                "again_reduction", "subitems_amount", "deposit_amount", "amount", "paid_amount", "unpaid_amount", "fee", "photos", "files", "score", "reply", "remark",
            ],
        )
        record_obj = EnrollRecord.objects.filter(id=pk)
        if not record_obj.first():
            return None, "没有找到id为" + str(pk) + "的记录"
        try:
            record_obj.update(**params)
        except Exception as e:
            return None, "修改异常:" + str(e)
        return None, None

    @staticmethod
    def record_del(pk, search_params=None):
        if pk:
            record_obj = EnrollRecord.objects.filter(id=pk)
        elif search_params and isinstance(search_params, dict):
            record_obj = EnrollRecord.objects.filter(**search_params)
        else:
            return None, "找不到要删除的数据"

        if not record_obj:
            return None, None
        try:
            record_obj.delete()
        except Exception as e:
            return None, "删除异常:" + str(e)
        return None, None



"""
@project: djangoModel->tool
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: CURD 工具
@created_time: 2022/9/15 14:14
"""

from django.core.paginator import Paginator, EmptyPage
from django.db import transaction

from xj_finance.services.finance_transacts_service import FinanceTransactsService
from ..models import Enroll, EnrollRecord
from ..service.clock_service import ClockService
from ..service.enroll_record_serivce import EnrollRecordServices
from ..service.enroll_subitem_record_service import EnrollSubitemRecordService
from ..service.subitem_service import SubitemService
from ..utils.custom_tool import format_params_handle


class EnrollServices:
    def __init__(self):
        pass

    @staticmethod
    def enroll_add(params, is_valuate=False, subitem=None):
        """
        报名表新增
        :param params:
        :param is_valuate: 是否使用计价，False直接使用传过来的值
        :return: data,err
        """
        params = format_params_handle(
            param_dict=params,
            filter_filed_list=[
                "thread_id", "user_id", "category_id", "trading_relate", "region_code", "occupy_room", "enroll_status_code", "min_number", "max_number", "min_count_apiece", "max_count_apiece",
                "enroll_rule_group", "price", "count", "unit", "fee", "reduction", "subitems_amount", "amount", "paid_amount", "unpaid_amount", "commision", "deposit",
                "hide_price", "hide_user", "has_repeat", "has_subitem", "has_audit", "has_vouch", "need_deposit", "need_imprest", "enable_pool",
                "pool_limit", "pool_stopwatch", "open_time", "close_time", "launch_time", "finish_time", "spend_time", "create_time", "update_time", "snapshot", "remark",
            ],
        )
        if is_valuate:
            pass

        try:
            instance = Enroll.objects.create(**params)
        except Exception as e:
            return None, str(e)

        return instance.to_json(), None

    @staticmethod
    def enroll_detail(enroll_id, simple_return=False):
        enroll_obj = Enroll.objects.filter(id=enroll_id)
        if not enroll_obj:
            return None, "不存的报名信息"
        enroll_detail = enroll_obj.first().to_json()
        if not simple_return:
            # 报名记录筛选出报名用户ID
            enroll_user_ids = list(EnrollRecord.objects.filter(enroll_id=enroll_id).values("user_id"))
            enroll_detail["enroll_user_ids"] = [i["user_id"] for i in enroll_user_ids]
            # 报名分项
            enroll_detail["subitem"], err = SubitemService.list({"enroll_id": enroll_id}, False)
            # 清单总数
            enroll_detail["detailed_count"] = 0
            if not err and isinstance(enroll_detail["subitem"], list) and len(enroll_detail["subitem"]) >= 1:
                enroll_detail["detailed_count"] = enroll_detail["subitem"][0]['count']

            clocker = ClockService()
            data, err = clocker.check_clock(enroll_id)
            enroll_detail.update(data)
        return enroll_detail, None

    @staticmethod
    def enroll_list(params, filter_fields=None, need_pagination=True):
        # 字段处理
        size = params.pop('size', 10)
        page = params.pop('page', 1)
        filter_fields_list = filter_fields.split(";") if filter_fields else []
        params['enroll_status_code'] = params['enroll_status_code'].split(";") if params.get("enroll_status_code") else None
        params = format_params_handle(
            param_dict=params,
            filter_filed_list=[
                "id", "thread_id", "category_id", "thread_id_list", "trading_relate", "region_code", "spend_time_start", "enroll_status_code",
                "spend_time_end", "create_time_start", "create_time_end", "finish_time_start", "finish_time_end",
                "open_time_start", "open_time_end", "enroll_status_code", "has_subitem"
            ],
            alias_dict={
                "spend_time_start": "spend_time__gte", "spend_time_end": "spend_time__lte", "create_time_start": "create_time__gte", "create_time_end": "create_time__lte",
                "finish_time_start": "finish_time__gte", "open_time_start": "open_time__gte", "open_time_end": "open_time__lte", "enroll_classify_value": "classify__value",
                "enroll_category_value": "category__value", "enroll_status_code": "enroll_status_code__in", "thread_id_list": "thread_id__in"
            }
        )
        # 全部展示
        enroll_obj = Enroll.objects.filter(**params).values(*filter_fields_list)
        if not need_pagination:
            return list(enroll_obj), None
        # 分页展示
        paginator = Paginator(enroll_obj, size)
        try:
            enroll_obj = paginator.page(page)
        except EmptyPage:
            enroll_obj = paginator.page(paginator.num_pages)
        except Exception as e:
            return None, f'{str(e)}'
        return {'total': paginator.count, "page": page, "size": size, 'list': list(enroll_obj.object_list)}, None

    @staticmethod
    def enroll_own_list(params):
        size = params.pop('size', 10)
        page = params.pop('page', 1)
        enroll_obj = Enroll.objects.filter(**params).values()
        paginator = Paginator(enroll_obj, size)
        try:
            enroll_obj = paginator.page(page)
        except EmptyPage:
            enroll_obj = paginator.page(paginator.num_pages)
        except Exception as e:
            return None, f'{str(e)}'
        return {'total': paginator.count, "page": page, "size": size, 'list': list(enroll_obj.object_list)}, None

    @staticmethod
    def enroll_edit(params=None, enroll_id=None, search_param=None):
        if not enroll_id and not search_param:
            return None, "无法找到要修改数据，请检查参数"
        enroll_id = params.pop("enroll_id", None) or enroll_id
        params = format_params_handle(
            param_dict=params,
            filter_filed_list=[
                "thread_id", "category_id", "trading_relate", "region_code", "occupy_room", "enroll_status_code", "min_number", "max_number", "min_count_apiece", "max_count_apiece",
                "enroll_rule_group", "price", "count", "unit", "fee", "reduction", "subitems_amount", "amount", "paid_amount", "unpaid_amount", "commision", "deposit",
                "bid_mode", "ticket", "hide_price", "hide_user", "has_repeat", "has_subitem", "has_audit", "has_vouch", "need_deposit", "need_imprest", "enable_pool",
                "pool_limit", "pool_stopwatch", "open_time", "close_time", "launch_time", "finish_time", "spend_time", "create_time", "update_time", "snapshot", "remark",
            ],
        )
        enroll_obj = Enroll.objects
        if enroll_id:
            enroll_obj = enroll_obj.filter(id=enroll_id)
        if search_param:
            enroll_obj = enroll_obj.filter(**search_param)
        if not enroll_obj:
            return None, "找不到可修改的数据"

        try:
            enroll_obj.update(**params)
        except Exception as e:
            return None, "修改异常:" + str(e)
        return enroll_obj.first().to_json(), None

    @staticmethod
    def enroll_delete(enroll_id):
        enroll_obj = Enroll.objects.filter(id=enroll_id)
        if not enroll_obj:
            return None, None
        try:
            enroll_obj.delete()
        except Exception as e:
            return None, "删除异常:" + str(e)
        return None, None

    @staticmethod
    def enroll(request_params=None):
        """报名记录联合添加报名记录和分项记录"""
        sid = transaction.savepoint()
        # 信息主表
        main_record_params = format_params_handle(
            param_dict=request_params,
            filter_filed_list=[
                "id", "enroll_id", "user_id", "enroll_auth_status_id", "enroll_pay_status_id", "enroll_status_code", "create_time", "price", "deposit", "count",
                "main_amount", "coupon_amount", "again_reduction", "subitems_amount", "deposit_amount", "amount", "paid_amount", "unpaid_amount", "fee", "photos",
                "files", "score", "reply", "remark",
            ]
        )
        main_data, err = EnrollRecordServices.record_add(main_record_params)
        if err:
            transaction.savepoint_rollback(sid)
            return None, err

        # 分项表
        subitem_params = format_params_handle(
            param_dict=request_params,
            filter_filed_list=["enroll_record_id", "enroll_subitem_id", "user_id", "subitem_price", "subitem_count", "subitem_amount", ],
            alias_dict={"subitem_price": "price", "subitem_count": "count"}
        )

        enroll_record_id = main_data.get("id", None)
        if not enroll_record_id:
            transaction.savepoint_rollback(sid)
            return None, err
        subitem_params["enroll_record_id"] = enroll_record_id

        subitem_data, err = EnrollSubitemRecordService.add(subitem_params)
        if err:
            transaction.savepoint_rollback(sid)
            return None, err

        # 添加定时器
        clocker = ClockService()
        clocker.add_clock(enroll_id=request_params.get("enroll_id"), user_id=request_params.get("user_id"))
        transaction.clean_savepoints()
        return None, err

    @staticmethod
    def pay_call_back(order_no):
        finance_transact_data, err = FinanceTransactsService.detail(order_no=order_no)
        # print("finance_transact_data:", finance_transact_data)
        if err:
            return None, err
        user_id = finance_transact_data.get("account_id")
        enroll_id = finance_transact_data.get("enroll_id")
        print("支付回调报名", "user_id:", user_id, "enroll_id:", enroll_id)
        enroll_obj = Enroll.objects.filter(id=enroll_id, user_id=user_id)
        if not enroll_obj:  # 不知互动发起者，支付的订单
            return None, None
        enroll_obj.update(paid_amount=finance_transact_data.get("outgo", 0), enroll_status_code=43)

        enroll_record = EnrollRecord.objects.filter(enroll_id=enroll_id)
        enroll_record.update(enroll_status_code=43)
        return None, None



# encoding: utf-8
"""
@project: djangoModel->enroll_statistics
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 统计接口
@created_time: 2022/10/31 11:07
"""
from django.views.decorators.http import require_http_methods
from rest_framework.views import APIView

from ..service.enroll_statistics_service import EnrollStatisticsService
from ..utils.custom_response import util_response
from ..utils.custom_tool import request_params_wrapper


class EnrollStatisticsAPI(APIView):
    @require_http_methods(['GET'])
    @request_params_wrapper
    def statistics(self, *args, request_params=None, **kwargs):
        data, err = EnrollStatisticsService.statistics_by_day()
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)



from django.db import transaction
from django.views.decorators.http import require_http_methods
from rest_framework.views import APIView

from xj_user.utils.user_wrapper import user_authentication_wrapper, user_authentication_force_wrapper
from ..service.enroll_subitem_record_service import EnrollSubitemRecordService
from ..utils.custom_response import util_response
from ..utils.custom_tool import parse_data


class SubitemRecordApis(APIView):

    @require_http_methods(['POST'])
    @user_authentication_force_wrapper
    def add(self, *args, user_info=None, **kwargs, ):
        params = parse_data(self)
        params["user_id"] = user_info.get("user_id")
        data, err = EnrollSubitemRecordService.add(params)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @require_http_methods(['POST'])
    @user_authentication_force_wrapper
    def batch_add(self, *args, user_info=None, **kwargs, ):
        sid = transaction.savepoint()
        params = parse_data(self)
        try:
            for param in params:
                data, err = EnrollSubitemRecordService.add(param)
                if err:
                    transaction.savepoint_rollback(sid)
                    return util_response(err=1000, msg=err)

            transaction.clean_savepoints()
            return util_response()
        except Exception as e:
            return util_response(msg=str(e))

    @require_http_methods(['GET'])
    @user_authentication_wrapper
    def list(self, *args, **kwargs, ):
        request_params = parse_data(self)
        need_pagination = request_params.get("need_pagination", 0)
        need_pagination = int(need_pagination)
        data, err = EnrollSubitemRecordService.list(params=request_params, need_pagination=need_pagination)
        if err:
            return util_response(err=1000, msg=data)
        return util_response(data=data)

    @require_http_methods(['PUT'])
    def edit(self, *args, **kwargs, ):
        params = parse_data(self)
        subitem_id = params.pop("id", None) or kwargs.pop("pk", None)
        data, err = EnrollSubitemRecordService.edit(params, subitem_id)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @require_http_methods(['DELETE'])
    def delete(self, *args, **kwargs, ):
        params = parse_data(self)
        pk = params.pop("id", None) or kwargs.pop("pk", None)
        data, err = EnrollSubitemRecordService.delete(pk)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)



# encoding: utf-8
"""
@project: djangoModel->rule_service
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 报名规则表
@created_time: 2022/9/19 15:54
"""
from django.core.paginator import Paginator

from ..models import EnrollRuleValuate, EnrollRuleGroup
from ..utils.custom_tool import format_params_handle


class RuleValueService():
    @staticmethod
    def list(params):
        size = params.pop('size', 10)
        page = params.pop('page', 1)
        params = format_params_handle(
            param_dict=params,
            filter_filed_list=["id", "enroll_rule_group_id", "name", "type", "field", "expression_string", ],
        )
        try:
            fetch_obj = EnrollRuleValuate.objects.filter(**params).values()
            paginator = Paginator(fetch_obj, size)
            page_obj = paginator.page(page)
            data = {'total': paginator.count, "size": size, 'page': page, 'list': list(page_obj.object_list)}
            return data, None
        except Exception as e:
            return None, str(e)

    @staticmethod
    def edit(params, rule_value_id):
        rule_value_id = params.pop("rule_value_id", None) or rule_value_id
        params = format_params_handle(
            param_dict=params,
            filter_filed_list=["rule_value_id", "enroll_rule_group_id", "name", "type", "field", "expression_string", ],
        )
        enroll_obj = EnrollRuleValuate.objects.filter(id=rule_value_id)
        if not enroll_obj:
            return None, None
        try:
            enroll_obj.update(**params)
        except Exception as e:
            return None, "修改异常:" + str(e)
        return None, None

    @staticmethod
    def delete(rule_value_id):
        enroll_obj = EnrollRuleValuate.objects.filter(id=rule_value_id)
        if not enroll_obj:
            return None, None
        try:
            enroll_obj.delete()
        except Exception as e:
            return None, "删除异常:" + str(e)
        return None, None

    @staticmethod
    def add(params):
        params = format_params_handle(
            param_dict=params,
            filter_filed_list=["rule_value_id", "enroll_rule_group_id", "name", "type", "field", "expression_string", ],
        )
        try:
            EnrollRuleValuate.objects.create(**params)
        except Exception as e:
            return None, str(e)

        return None, None

    @staticmethod
    def group_list(request_params):
        print(request_params)
        params = format_params_handle(
            param_dict=request_params,
            filter_filed_list=["id", "classifyt_id", "category_id", "rule_group"],
            alias_dict={"rule_group": "rule_group__contains"}
        )
        res_obj = EnrollRuleGroup.objects.filter(**params)
        if res_obj:
            return res_obj.to_json(), None
        else:
            return None, None



# encoding: utf-8
"""
@project: djangoModel->enroll_validator
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 报名验证器
@created_time: 2022/9/17 15:50
"""

from django import forms

from ..utils.validator import Validator


class RecordValidator(Validator):
    enroll_id = forms.IntegerField(
        required=True,
        error_messages={
            "required": "enroll_id 必填 必填",
        },
    )

    # 报名开始与截至时间
    user_id = forms.IntegerField(
        required=True,
        error_messages={
            "required": "user_id 必填 必填",
        },
    )



# encoding: utf-8
"""
@project: djangoModel->valuation_service
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 报名计时服务
@created_time: 2022/10/13 9:40
"""
from django.core.paginator import Paginator, EmptyPage
from django.db import transaction

from ..models import EnrollSubitemRecord
from ..utils.custom_tool import format_params_handle


class EnrollSubitemRecordService:

    @staticmethod
    def add(params):
        params = format_params_handle(
            param_dict=params,
            filter_filed_list=["enroll_record_id", "enroll_subitem_id", "user_id", "price", "count", "subitem_amount", ],
        )
        try:
            instance = EnrollSubitemRecord.objects.create(**params)
            return instance.to_json(), None
        except Exception as e:
            return None, str(e)

    @staticmethod
    def list(params, need_pagination=True):
        # 字段处理
        size = params.pop('size', 10)
        page = params.pop('page', 1)
        params['enroll_status_code'] = params['enroll_status_code'].split(";") if params.get("enroll_status_code") else None
        params = format_params_handle(
            param_dict=params,
            filter_filed_list=["enroll_record_id", "enroll_subitem_id", "user_id", "price", "count", "subitem_amount", ],
            alias_dict={"enroll_status_code": "enroll_status_code__in"}
        )
        # 全部展示
        query_obj = EnrollSubitemRecord.objects.filter(**params).values()
        if not need_pagination:
            return list(query_obj), None

        paginator = Paginator(query_obj, size)
        # 分页展示
        try:
            paginator_obj = paginator.page(page)
        except EmptyPage:
            paginator_obj = paginator.page(paginator.num_pages)
        except Exception as e:
            print("e", e)
            return None, f'{str(e)}'

        return {'total': paginator.count, "page": page, "size": size, 'list': list(paginator_obj.object_list)}, None

    @staticmethod
    def edit(params, pk):
        pk = params.pop("id", None) or pk
        params = format_params_handle(
            param_dict=params,
            filter_filed_list=["enroll_record_id", "enroll_subitem_id", "price", "count", "subitem_amount", ],
        )
        query_obj = EnrollSubitemRecord.objects.filter(id=pk)
        if not query_obj:
            return None, "没有可修改的数据"
        try:
            instance = query_obj.update(**params)
        except Exception as e:
            return None, "修改异常:" + str(e)
        return None, None

    @staticmethod
    def delete(pk, search_params=None):
        """取消报名"""
        if not search_params:
            subrecord_obj = EnrollSubitemRecord.objects.filter(id=pk)
        else:
            try:
                subrecord_obj = EnrollSubitemRecord.objects.filter(**search_params)
            except Exception as e:
                return None, "搜索参数不正确"

        if not subrecord_obj:
            return None, "没有可修改得到数据"

        try:
            subrecord_obj.delete()
        except Exception as e:
            return None, "删除异常:" + str(e)
        return None, None

    @staticmethod
    def batch_copy(pk=None, copy_params=None, copy_num=None, ):
        """
        批量复制报名分项记录
        :param copy_params: 复制插入的参数
        :param pk: 搜索ID主键
        :param copy_count: 复制参数
        :return: data,err
        """
        if not copy_num:
            return None, "拷贝的分数不能为空，至少为1分"

        if not pk and not copy_params:
            return None, "找不到可以复制的记录"

        # 开始事务
        sid = transaction.savepoint()
        try:
            if not copy_params or isinstance(copy_params, dict):
                copy_params_obj = EnrollSubitemRecord.objects.filter(id=pk)
                if not copy_params_obj:
                    return None, "找不到可以复制的记录"
                copy_params = copy_params_obj.first().to_json()

            # 遍历插入数据
            for i in range(copy_num):
                EnrollSubitemRecord.objects.create(**copy_params)

            # 清除保存点
            transaction.clean_savepoints()
            return None, None

        except Exception as e:
            print("err:", str(e))
            transaction.savepoint_rollback(sid)
            return None, str(e)



# encoding: utf-8
"""
@project: djangoModel->rule_api
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis:  计价 规则API
@created_time: 2022/9/20 13:20
"""
from django.views.decorators.http import require_http_methods
from rest_framework.views import APIView

from ..service.rule_service import RuleValueService
from ..utils.custom_response import util_response
from ..utils.custom_tool import parse_data, request_params_wrapper
from ..validator.rule_validator import RuleValidator


class RuleAPI(APIView):
    @require_http_methods(['GET'])
    def list(self, *args, **kwargs, ):
        params = parse_data(self)
        data, err = RuleValueService.list(params=params)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @require_http_methods(['POST'])
    def add(self, *args, **kwargs, ):
        params = parse_data(self)
        # 表单数据验证
        is_valid, error = RuleValidator(params).validate()
        if not is_valid:
            return util_response(err=1000, msg=error)
        # 添加数据
        data, err = RuleValueService.add(params)
        if err:
            return util_response(err=1001, msg=err)
        return util_response(data=data)

    @require_http_methods(['PUT'])
    def edit(self, *args, **kwargs, ):
        params = parse_data(self)
        rule_value_id = kwargs.get("rule_value_id") or params.pop("rule_value_id") or None
        if not rule_value_id:
            return util_response(err=1000, msg="参数错误:enroll_id不可以为空")
        data, err = RuleValueService.edit(params, rule_value_id)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @require_http_methods(['DELETE'])
    def delete(self, *args, **kwargs, ):
        params = parse_data(self)
        rule_value_id = kwargs.get("rule_value_id") or params.pop("rule_value_id") or None
        if not rule_value_id:
            return util_response(err=1000, msg="参数错误:enroll_id不可以为空")
        data, err = RuleValueService.edit(params, rule_value_id)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    # 规则分组列表
    @request_params_wrapper
    def group_list(self, *args, request_params, **kwargs):
        data, err = RuleValueService.group_list(request_params)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)



from django.db import transaction
from django.views.decorators.http import require_http_methods
from rest_framework.views import APIView

from xj_thread.services.thread_item_service import ThreadItemService
from xj_thread.services.thread_list_service import ThreadListService
from xj_user.services.user_detail_info_service import DetailInfoService
from xj_user.utils.user_wrapper import user_authentication_force_wrapper, user_authentication_wrapper
from ..join_list import JoinList
from ..service.clock_service import ClockService
from ..service.enroll_record_serivce import EnrollRecordServices
from ..service.enroll_services import EnrollServices
from ..service.enroll_subitem_record_service import EnrollSubitemRecordService
from ..service.subitem_service import SubitemService
from ..utils.custom_response import util_response
from ..utils.custom_tool import parse_data, format_params_handle, request_params_wrapper, filter_result_field


class EnrollAPI(APIView):

    @require_http_methods(['POST'])
    @user_authentication_force_wrapper
    def add(self, *args, user_info, **kwargs, ):
        # print("user_info:", user_info)
        params = parse_data(self)
        # 表单数据验证
        # is_valid, error = EnrollValidator(params).validate()
        # if not is_valid:
        #     return util_response(err=1000, msg=error)

        # 字段验证处理
        user_id = user_info.get("user_id")
        detailed_count = params.pop("detailed_count", 0)  # 项目清单数量
        params.setdefault("subitem", [])  # 报名分项
        params.setdefault("user_id", user_id)  # 用户ID
        is_valuate = params.pop("is_valuate", None)
        thread_params = format_params_handle(
            param_dict=params,
            remove_filed_list=[
                # 报名表字段
                "trading_relate", "region_code", "occupy_room", "enroll_status_code", "min_number", "max_number", "min_count_apiece", "max_count_apiece",
                "enroll_rule_group", "price", "count", "unit", "fee", "reduction", "subitems_amount", "amount", "paid_amount", "unpaid_amount", "commision",
                "deposit", "hide_price", "hide_user", "has_repeat", "has_subitem", "has_audit", "need_vouch", "need_deposit", "need_imprest", "enable_pool",
                "pool_limit", "pool_stopwatch", "open_time", "close_time", "launch_time", "finish_time", "spend_time", "create_time",
                "update_time", "snapshot", "remark",
                # 报名分项字段
                "subitem"
            ],
            alias_dict={"thread_remark": "remark"}
        )
        if not thread_params:
            return util_response(err=1001, msg="请填写项目基本信息")

        main_params = format_params_handle(
            param_dict=params,
            filter_filed_list=[
                "thread_id", "user_id", "category_id", "trading_relate", "region_code", "occupy_room", "enroll_status_code", "min_number", "max_number",
                "min_count_apiece", "max_count_apiece", "enroll_rule_group", "price", "count", "unit", "fee", "reduction", "subitems_amount",
                "amount", "paid_amount", "unpaid_amount", "commision", "deposit", "hide_price", "hide_user", "has_repeat", "has_subitem", "has_audit",
                "need_vouch", "need_deposit", "need_imprest", "enable_pool", "pool_limit", "pool_stopwatch", "open_time", "close_time", "launch_time",
                "finish_time", "spend_time", "create_time", "update_time", "snapshot", "remark"
            ],
        )
        main_params.setdefault("snapshot", thread_params)  # 因为信息表需要调用服务，所以存一组信息表的快照
        main_params["count"] = len(params.get("subitem", []))
        sid = transaction.savepoint()

        # 信息表联调
        thread_id = thread_params.get("thread_id")
        if not thread_id:
            data, err = ThreadItemService.add(thread_params)
        else:
            data, err = ThreadItemService.edit(thread_params, thread_id)
        if err:
            transaction.savepoint_rollback(sid)
            return util_response(err=1002, msg="信息添加错误：" + err)

        # 报名主表添加
        thread_id = thread_id or data['id']
        main_params.setdefault("thread_id", thread_id or data['id'])
        main_params.setdefault("enroll_status_code", 242)
        main_instance, err = EnrollServices.enroll_add(main_params, is_valuate)
        if err:
            transaction.savepoint_rollback(sid)
            return util_response(err=1003, msg=err)

        # 报名分项联动
        for item_params in params.get("subitem", []):
            item_params.setdefault("enroll_id", main_instance.get('id'))
            item_params.setdefault("enroll_subitem_status_code", 242)
            # print("item_params", item_params)
            if detailed_count:
                item_params["count"] = detailed_count
            data, err = SubitemService.add(item_params)
            if err:
                transaction.savepoint_rollback(sid)
                return util_response(err=1004, msg=err)

        transaction.clean_savepoints()  # 清除保存点
        return util_response(data=main_instance)

    @require_http_methods(['GET'])
    def detail(self, *args, **kwargs, ):
        params = parse_data(self)
        filter_fields = params.pop("filter_fields", None)
        remove_fields = params.pop("remove_fields", None)
        enroll_id = kwargs.get("enroll_id") or params.pop("enroll_id") or None
        if not enroll_id:
            return util_response(err=1000, msg="参数错误:enroll_id不可以为空")
        data, err = EnrollServices.enroll_detail(enroll_id)
        if err:
            return util_response(err=1001, msg=err)

        # 用户数据拼接
        # todo 隐患就是一个活动报名人数非常多的时候 下面服务效率会非常低,当数据连比较大的时候注释下面这一行仅仅返回用户ID
        users = DetailInfoService.get_list_detail({}, data["enroll_user_ids"])
        data["enroll_user_info"] = [{"user_id": i["user_id"], 'nickname': i["nickname"], 'user_name': i["user_name"], 'full_name': i["full_name"]} for i in users]

        # 信息表 数据拼接
        thread_list, err = ThreadListService.search([data.get("thread_id")])
        if err:
            return util_response(err=1002, msg=err)
        data = JoinList([data], thread_list, "thread_id", "id").join()
        data = data[0] if not len(data) > 1 else data
        data["price"] = round(data["price"], 2)
        # 字段过滤
        filter_fields = filter_fields.split(";") if filter_fields else None
        remove_fields = remove_fields.split(";") if remove_fields else None
        data = format_params_handle(data, filter_filed_list=filter_fields, remove_filed_list=remove_fields, is_remove_null=False)
        # 响应数据
        if err:
            return util_response(err=1003, msg=err)
        return util_response(data=data)

    @require_http_methods(['GET'])
    @request_params_wrapper
    @user_authentication_wrapper
    def list(self, *args, request_params=None, user_info=None, **kwargs, ):
        # 如果需要 信息表字典筛选，thread_id_list 放入thread_id即可
        filter_fields = request_params.get("filter_fields", None)
        data, err = EnrollServices.enroll_list(params=request_params)
        if err:
            return util_response(err=1000, msg=err)
        # 合并thread模块数据
        id_list = [i['thread_id'] for i in data['list'] if i]
        thread_list, err = ThreadListService.search(id_list)
        data['list'] = JoinList(data['list'], thread_list, "thread_id", "id").join()
        # 小数点四舍五入
        for i in data['list']:
            i["price"] = round(i["price"], 2)

        # 结果集字段过滤
        filter_fields = filter_fields.split(";") if filter_fields else None
        data["list"] = filter_result_field(result_list=data["list"], filter_filed_list=filter_fields)
        # 数据返回
        if err:
            return util_response(err=1001, msg=err)
        return util_response(data=data)

    @require_http_methods(['GET'])
    @request_params_wrapper
    @user_authentication_force_wrapper
    def own_list(self, *args, request_params=None, user_info=None, **kwargs):
        filter_fields = request_params.get("filter_fields", None)
        user_id = user_info.get("user_id")
        params = {"user_id": user_id, "page": request_params.get("page", 1), "size": request_params.get("page", 20)}
        data, err = EnrollServices.enroll_own_list(params)
        if err:
            return util_response(err=1000, msg=err)
        # 合并thread模块数据
        id_list = [i['thread_id'] for i in data['list'] if i]
        thread_list, err = ThreadListService.search(id_list)
        data['list'] = JoinList(data['list'], thread_list, "thread_id", "id").join()

        filter_fields = filter_fields.split(";") if filter_fields else None
        data["list"] = filter_result_field(result_list=data["list"], filter_filed_list=filter_fields)
        # 数据返回
        if err:
            return util_response(err=1001, msg=err)
        return util_response(data=data)

    @require_http_methods(['PUT'])
    def edit(self, *args, **kwargs, ):
        # 参数处理
        params = parse_data(self)
        enroll_id = kwargs.get("enroll_id") or params.pop("enroll_id") or None
        if not enroll_id:
            return util_response(err=1000, msg="参数错误:enroll_id不可以为空")
        thread_params = format_params_handle(
            param_dict=params,
            remove_filed_list=[
                "trading_relate", "user_id", "region_code", "occupy_room", "enroll_status_code", "min_number", "max_number",
                "min_count_apiece", "max_count_apiece", "enroll_rule_group", "price", "count", "unit", "fee", "reduction", "subitems_amount", "amount",
                "paid_amount", "unpaid_amount", "commision", "deposit", "hide_price", "hide_user", "has_repeat", "has_subitem", "has_audit", "need_vouch",
                "need_deposit", "need_imprest", "enable_pool", "pool_limit", "pool_stopwatch", "open_time", "close_time", "launch_time", "finish_time",
                "spend_time", "create_time", "update_time", "snapshot", "remark",
            ],
            alias_dict={"thread_remark": "remark"}
        )

        main_params = format_params_handle(
            param_dict=params,
            filter_filed_list=[
                "thread_id", "category_id", "trading_relate", "region_code", "occupy_room", "enroll_status_code", "min_number", "max_number",
                "min_count_apiece", "max_count_apiece", "enroll_rule_group", "price", "count", "unit", "fee", "reduction", "subitems_amount", "amount",
                "paid_amount", "unpaid_amount", "commision", "deposit", "hide_price", "hide_user", "has_repeat", "has_subitem", "has_audit", "need_vouch",
                "need_deposit", "need_imprest", "enable_pool", "pool_limit", "pool_stopwatch", "open_time", "close_time", "launch_time", "finish_time",
                "spend_time", "create_time", "update_time", "snapshot", "remark",
            ]
        )
        main_params.setdefault("snapshot", thread_params)  # 因为信息表需要调用服务，所以存一组信息表的快照

        # 报名主表修改
        instance, err = EnrollServices.enroll_edit(main_params, enroll_id)
        if err:
            return util_response(err=1001, msg=err)

        # 信息表修改
        thread_id = params.pop("thread_id", None) or instance.get("thread_id", None)
        if thread_id and thread_params:
            data, err = ThreadItemService.edit(thread_params, thread_id)
            if err:
                return util_response(err=1002, msg=err)

        # 联动分项数量修改
        detailed_count = params.pop("detailed_count", None)
        if detailed_count:
            data, err = SubitemService.batch_edit({"count": detailed_count}, enroll_id=enroll_id)
            if err:
                return util_response(err=1003, msg=err)

        return util_response(data=instance)

    @require_http_methods(['DELETE'])
    def delete(self, *args, **kwargs, ):
        params = parse_data(self)
        enroll_id = kwargs.get("enroll_id") or params.pop("enroll_id") or None
        if not enroll_id:
            return util_response(err=1000, msg="参数错误:enroll_id不可以为空")
        data, err = EnrollServices.enroll_delete(enroll_id)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @require_http_methods(['POST'])
    @request_params_wrapper
    @user_authentication_wrapper
    def enroll(self, *args, request_params=None, user_info=None, **kwargs):
        sid = transaction.savepoint()
        # 用户id获取
        request_params.setdefault("user_id", user_info.get("user_id", None))
        if not request_params.get("user_id"):
            return util_response(err=1000, msg="user_id 必填")
        # is_can_add = EnrollRecordServices.check_can_add(enroll_id=request_params.get("enroll_id"), user_id=user_info.get("user_id"))
        # if not is_can_add:
        #     return util_response(err=1001, msg="无法继续报名，当前人数已满")

        # 信息主表
        main_record_params = format_params_handle(
            param_dict=request_params,
            filter_filed_list=[
                "id", "enroll_id", "user_id", "enroll_auth_status_id", "enroll_pay_status_id", "enroll_status_code", "create_time", "price", "deposit", "count",
                "main_amount", "coupon_amount", "again_reduction", "subitems_amount", "deposit_amount", "amount", "paid_amount", "unpaid_amount", "fee", "photos",
                "files", "score", "reply", "remark",
            ]
        )
        main_data, err = EnrollRecordServices.record_add(main_record_params)
        if err:
            transaction.savepoint_rollback(sid)
            return util_response(err=1001, msg=err)

        # 分项表
        subitem_params = format_params_handle(
            param_dict=request_params,
            filter_filed_list=["enroll_record_id", "enroll_subitem_id", "user_id", "subitem_price", "subitem_count", "subitem_amount", ],
            alias_dict={"subitem_price": "price", "subitem_count": "count"}
        )

        enroll_record_id = main_data.get("id", None)
        if not enroll_record_id:
            transaction.savepoint_rollback(sid)
            return util_response(err=1002, msg=err)
        subitem_params["enroll_record_id"] = enroll_record_id

        subitem_data, err = EnrollSubitemRecordService.add(subitem_params)
        if err:
            transaction.savepoint_rollback(sid)
            return util_response(err=1003, msg=err)

        transaction.clean_savepoints()
        # 添加定时器
        clocker = ClockService()
        clocker.add_clock(enroll_id=request_params.get("enroll_id"), user_id=request_params.get("user_id"))
        return util_response()

    @require_http_methods(['DELETE'])
    @request_params_wrapper
    @user_authentication_wrapper
    def un_enroll(self, *args, request_params=None, user_info=None, **kwargs):
        # 用户id获取
        user_id = request_params.get("user_id") or user_info.get("user_id")
        enroll_id = request_params.get("enroll_id")
        if not user_id or not enroll_id:
            return util_response(err=1000, msg="参数错误")

        data, err = EnrollRecordServices.record_del(None, {"user_id": user_id, "enroll_id": enroll_id})

        return util_response()

    @require_http_methods(['GET'])
    @request_params_wrapper
    def enroll_pay_callback(self, *args, request_params=None, **kwargs):
        # 用户id获取
        order_no = request_params.get("order_no", None)
        if not order_no:
            return util_response(err=1000, msg="order_no 不能为空")

        data, err = EnrollServices.pay_call_back(order_no)
        if err:
            return util_response(err=1001, msg=err)
        return util_response(msg="回调成功")



# encoding: utf-8
"""
@project: djangoModel->subitem_service
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 报名分项记录
@created_time: 2022/10/15 12:38
"""

from django.core.paginator import Paginator
from django.db.models import F

from ..models import EnrollSubitem, EnrollSubitemExtendField, Enroll, EnrollSubitemRecord
from ..service.subitem_extend_service import input_convert, output_convert
from ..utils.custom_tool import format_params_handle


class SubitemService:
    @staticmethod
    def add(params):
        enroll_id = params.get("enroll_id")
        if not enroll_id:
            return None, "请填写报名ID"

        params = input_convert(
            params_dict=params,
            enroll_id=enroll_id
        )
        # print("SubitemService.params", params)
        try:
            instance = EnrollSubitem.objects.create(**params)
            return instance.to_json(), None
        except Exception as e:
            return None, str(e)

    @staticmethod
    def detail(pk=None):
        # 参数验证
        subitem_obj = EnrollSubitem.objects.annotate(category_id=F("enroll__category_id")).filter(id=pk).first()
        if not subitem_obj:
            return None, "找不到ID为" + str(pk) + "的数据"
        subitem_dict = subitem_obj.to_json()
        subitem_list = output_convert([subitem_dict])
        return subitem_list[0], None

    @staticmethod
    def list(params, is_pagination=True):
        size = params.pop('size', 10)
        page = params.pop('page', 1)
        # 字段过滤
        user_id = params.get("user_id", None)
        params = format_params_handle(
            param_dict=params,
            filter_filed_list=["id", "category_id", "enroll_subitem_status_code", "enroll_id", "name", "price", "count", "unit", "description", "remark"],
            alias_dict={"name": "name__contains"}
        )
        try:
            fetch_obj = EnrollSubitem.objects.annotate(category_id=F("enroll__category_id")).filter(**params).values()
            if not is_pagination:
                result_list = output_convert(list(fetch_obj))
                result = []
                for item in result_list:
                    record = SubitemService.check_has_enroll(item.get("id"), user_id)
                    item["is_enroll"] = 1 if record else 0
                    item["record"] = record.to_json() if record else {}
                    result.append(item)
                return result, None

            paginator = Paginator(fetch_obj, size)
            page_obj = paginator.page(page)
            result_list = list(page_obj.object_list)
            result_list = output_convert(result_list)
            # 报名分项是否存在报名记录
            for item in result_list:
                record = SubitemService.check_has_enroll(item.get("id"), user_id)
                item["is_enroll"] = 1 if record else 0
                item["record"] = record.to_json() if record else {}

            data = {'total': paginator.count, "size": size, 'page': page, 'list': result_list}
            return data, None
        except Exception as e:
            print(e)
            return [], str(e)

    @staticmethod
    def check_has_enroll(subitem_id, user_id=None):
        obj = EnrollSubitemRecord.objects.filter(enroll_subitem_id=subitem_id).exclude(enroll_subitem_status_code=124)
        if user_id:
            return obj.filter(user_id=user_id).first()
        return None

    @staticmethod
    def edit(params, subitem_id=None):
        # 参数验证
        subitem_id = subitem_id or params.pop("id", None) or 0
        subitem_obj = EnrollSubitem.objects.filter(id=subitem_id)
        subitem_obj_first = subitem_obj.first()
        if not subitem_obj_first:
            return None, "找不到ID为" + str(subitem_id) + "的数据"
        # 开始修改
        try:
            enroll_id = params.get("enroll_id") or subitem_obj_first.to_json().get("enroll_id")
            # 参数解析
            params = input_convert(
                params_dict=params,
                enroll_id=enroll_id
            )
            subitem_obj.update(**params)
        except Exception as e:
            return None, "修改异常:" + str(e)
        return None, None

    # 批量修改
    @staticmethod
    def batch_edit(params, enroll_id=None):
        enroll_id = enroll_id or params.pop("enroll_id", None)
        if not enroll_id:
            return None, "请填写报名ID"

        # 参数根据类别转化
        params = input_convert(
            params_dict=params,
            enroll_id=enroll_id
        )
        if not params:
            return None, "enroll_id不能为空，或者参数为空"

        subitem_enroll_obj = EnrollSubitem.objects.filter(enroll_id=enroll_id)
        if not subitem_enroll_obj:
            return None, "没有找到enroll_id为" + str(enroll_id) + "的报名分项"
        try:
            subitem_enroll_obj.update(**params)
        except Exception as e:
            return None, "修改参数错误:" + str(e)
        return None, None

    @staticmethod
    def delete(subitem_rule_id):
        subitem_enroll_obj = EnrollSubitem.objects.filter(id=subitem_rule_id)
        if not subitem_enroll_obj:
            return None, None
        try:
            subitem_enroll_obj.delete()
        except Exception as e:
            return None, "删除异常:" + str(e)
        return None, None

    @staticmethod
    def extend_field(params=None, is_pagination=True):
        validate_params = params if isinstance(params, dict) else {}

        size = validate_params.pop('size', 10)
        page = validate_params.pop('page', 1)

        filtered_params = format_params_handle(
            param_dict=validate_params,
            filter_filed_list=["id", "category_id", "field_index", "field", "label", "type", "config", "description", ],
            alias_dict={"field": "field__contains", "label": "label__contains"}
        )

        try:
            extend_obj = EnrollSubitemExtendField.objects.all()
            extend_obj_list = extend_obj.filter(**filtered_params).values()
            if not is_pagination:
                return list(extend_obj_list), None

            paginator = Paginator(extend_obj_list, size)
            paginator_obj_list = paginator.page(page)
            data = {'total': paginator.count, "size": size, 'page': page, 'list': list(paginator_obj_list.object_list)}
            return data, None
        except Exception as e:
            return [], "查询参数错误：" + str(e)

    @staticmethod
    def check_num(enroll_id):
        enroll_obj = Enroll.objects.filter(id=enroll_id).first()
        if not enroll_obj:
            return False
        enroll_count = enroll_obj.to_json().get("count")
        subitem_count = EnrollSubitem.objects.filter(enroll_id=enroll_id).count()
        if enroll_count > subitem_count:
            return True
        return False



# encoding: utf-8
"""
@project: djangoModel->clock_service
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 报名计时服务
@created_time: 2022/10/28 13:37
"""
from django.db.models import F
from django_redis import get_redis_connection

from ..models import EnrollRecord, Enroll, EnrollSubitem, EnrollSubitemRecord
from ..utils.custom_tool import deal_equally

ENROLL_CLOCK = "enroll-clock-ttl-{}"  # 倒计时
ENROLLED_HSET_LIST = "enroll-hset-list"


class ClockService:
    def __init__(self):
        self.conn = get_redis_connection()

    def add_clock(self, enroll_id=None, user_id=None):
        if not enroll_id or not user_id:
            return None, "enroll_id 和 user_id 必传"

        # 报名状态进入356 (已接单待上传)
        clock_key = ENROLL_CLOCK.format(enroll_id)
        has_clock = self.conn.get(clock_key)
        # print("has_clock:", has_clock)
        if not has_clock:
            self.conn.set(clock_key, user_id)
            self.conn.expire(clock_key, 600)

        # 记录报名用户
        enroll_users = self.conn.hget(ENROLLED_HSET_LIST, enroll_id)
        # print("enroll_users:", enroll_users)
        users_list = enroll_users.decode().split(";") if enroll_users else []
        users_list.append(str(user_id))
        users_list = list(set(users_list))

        users_list_str = ""
        for i in users_list:
            users_list_str = users_list_str + (";" if users_list_str else "") + i

        self.conn.hset(ENROLLED_HSET_LIST, enroll_id, users_list_str)
        self.conn.expire(ENROLLED_HSET_LIST, 660)
        return "ok", None

    # 闹钟是否停止
    def check_clock(self, enroll_id=None):
        if not enroll_id:
            return None, "enroll_i 必传且不能为空"
        clock_key = ENROLL_CLOCK.format(enroll_id)
        clock_ttl = self.conn.ttl(clock_key)
        return {"clock_ttl": clock_ttl}, None

    def check_end_clock(self):
        """定时脚本执行方法"""
        enroll_ids = self.conn.hkeys(ENROLLED_HSET_LIST)
        print("enroll_ids:", enroll_ids)
        for enroll_id in enroll_ids:
            # 时候报名结束
            is_active_clock = self.conn.get(ENROLL_CLOCK.format(enroll_id.decode()))
            if is_active_clock:
                continue

            # 计时结束则进行如下操作
            enroll_obj = Enroll.objects.filter(id=enroll_id)
            enroll_obj_first = enroll_obj.first()
            eroll_count = enroll_obj_first.to_json().get("count", 0) if enroll_obj_first else 0  # 需求份数
            if eroll_count == 0:
                continue

            enroll_record_obj = EnrollRecord.objects.filter(enroll_id=enroll_id)  # 主报名记录
            enroll_people_count = enroll_record_obj.count()  # 实际报名人数，报名人数按照主要的报名份数来计算
            subtems_obj = EnrollSubitem.objects.annotate(enroll_subitem_id=F("id")).annotate(subitem_amount=F("amount")).filter(enroll_id=enroll_id)  # 报名分项

            # 根据报名定时截至的时候报名人数处理报名
            if enroll_people_count == 0:  # 当前没有人报名，什么都不做，报名人数为0。
                continue
            elif enroll_people_count == 1:  # 一个人报名，生成多条记录
                enroll_record_obj.update(count=eroll_count)
                overplus = eroll_count - enroll_people_count
                if overplus <= 0:
                    continue
                # 把剩余的名额全部全部给这个人报名 TODO
                enroll_record_obj.update(count=eroll_count)
                if subtems_obj.count() == 0:  # 没有报名分项的的报名则跳过下面的逻辑
                    continue

                values = list(subtems_obj.values("price", "count", "enroll_subitem_id", "subitem_amount"))
                enroll_record = enroll_record_obj.first()
                sub_record_obj = EnrollSubitemRecord.objects.filter(enroll_record_id=enroll_record.id)
                sub_record_obj.delete()

                for i in values:
                    params = i
                    params["enroll_record_id"] = enroll_record_obj.first().id
                    EnrollSubitemRecord.objects.create(**params)

            elif enroll_people_count > 1:
                # 发牌平均分逻辑
                num_list = deal_equally(eroll_count, enroll_people_count)
                # 获取所有的报名用户记录ID与用户ID
                enroll_record_list = EnrollRecord.objects.filter(enroll_id=enroll_id).values("id", "user_id")
                enroll_record_ids = {i["id"]: i["user_id"] for i in enroll_record_list}
                # 修改主报名记录人数
                for enroll_record_id, num in zip(enroll_record_ids.keys(), num_list):
                    obj = EnrollRecord.objects.filter(id=enroll_record_id)
                    obj.update(count=num, enroll_status_code=356)

                # 获取报名分项数据
                subtems_obj = EnrollSubitem.objects.annotate(enroll_subitem_id=F("id")).annotate(subitem_amount=F("amount")).filter(enroll_id=enroll_id)  # 报名分项
                if subtems_obj.count() == 0:  # 没有报名分项的的报名则跳过下面的逻辑
                    continue
                values = list(subtems_obj.values("price", "count", "enroll_subitem_id", "subitem_amount"))

                # 创建报名分项记录
                enroll_record_id_list = []
                for i in values:
                    if not enroll_record_id_list:
                        enroll_record_id_list = list(enroll_record_ids.keys())
                    params = i
                    params["enroll_record_id"] = enroll_record_id_list.pop()
                    params["user_id"] = enroll_record_ids.get(params["enroll_record_id"], None)
                    params["enroll_subitem_status_code"] = 356
                    print("params:", params)
                    EnrollSubitemRecord.objects.create(**params)

            # 当超过一个人的时候，则仅仅改动报名住哪个太由后台人员指定报名
            enroll_obj.update(enroll_status_code=356)
            self.conn.hdel(ENROLLED_HSET_LIST, enroll_id)



# encoding: utf-8
"""
@project: djangoModel->tool
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: CURD 工具
@created_time: 2022/6/15 14:14
"""

import json
import random
from urllib.parse import parse_qs

from django.core.handlers.asgi import ASGIRequest
from django.core.handlers.wsgi import WSGIRequest
from rest_framework.request import Request
import xmltodict


def is_number(s):
    """识别任何语言的数字字符串"""
    try:
        float(s)
        return True
    except ValueError:
        pass

    try:
        import unicodedata
        unicodedata.numeric(s)
        return True
    except (TypeError, ValueError):
        pass

    return False


# json 结果集返回
def parse_json(result):
    if not result is None:
        if type(result) is str:
            try:
                result = json.loads(result.replace("'", '"').replace('\\r', "").replace('\\n', "").replace('\\t', "").replace('\\t', ""))
            except Exception as e:
                return result
        if type(result) is list:
            for index, value in enumerate(result):
                result[index] = parse_json(value)
        if type(result) is dict:
            for k, v in result.items():
                result[k] = parse_json(v)
    return result


# 字段筛选并替换成别名
def format_params_handle(param_dict, filter_filed_list=None, remove_filed_list=None, alias_dict=None, is_remove_null=True):
    """
    字段筛选并替换成别名
    :param param_dict: 参数值
    :param filter_filed_list: 字段白名单
    :param remove_filed_list: 字段黑名单
    :param alias_dict: 表明字典
    :param is_remove_null:  是否把带有None的值移除掉
    :return: param_dict
    """
    # 转换的数据类型不符合，直接返回出去
    if not isinstance(param_dict, dict):
        raise Exception("param_dict 必须是字典格式")

    # 类型判断 过滤字段
    if filter_filed_list and isinstance(filter_filed_list, list):
        param_dict = {k: v for k, v in param_dict.copy().items() if k in filter_filed_list and (not v is None or not is_remove_null)}

    # 类型判断， 剔除字段
    if remove_filed_list and isinstance(remove_filed_list, list):
        param_dict = {k: v for k, v in param_dict.copy().items() if not k in remove_filed_list and (not v is None or not is_remove_null)}

    # 类型判断 字段转换
    if alias_dict and isinstance(alias_dict, dict):
        param_dict = {alias_dict.get(k, k): v for k, v in param_dict.copy().items()}

    return param_dict


# 结果接字段过滤
def filter_result_field(result_list, filter_filed_list=None, remove_filed_list=None, alias_dict=None):
    # 转换的数据类型不符合，直接返回出去
    if filter_filed_list is None and remove_filed_list is None and alias_dict is None:
        return result_list
    result = []
    for item in result_list:
        row_res = {}
        # 类型判断 过滤字段
        if filter_filed_list and isinstance(filter_filed_list, list):
            row_res = {k: v for k, v in item.copy().items() if k in filter_filed_list}
        # 类型判断， 剔除字段
        if remove_filed_list and isinstance(remove_filed_list, list):
            row_res = {k: v for k, v in item.copy().items() if not k in remove_filed_list}
        # 类型判断 字段转换
        if alias_dict and isinstance(alias_dict, dict):
            row_res = {alias_dict.get(k, k): v for k, v in item.copy().items()}

        if row_res:
            result.append(row_res)

    return result


# 请求参数解析
def parse_data(request):
    # 解析请求参数 兼容 APIView与View的情况，View 没有request.data
    content_type = request.META.get('CONTENT_TYPE', "").split(";")[0]
    method = request.method
    if content_type == "text/plain" or method == "GET":
        try:
            body = request.body.decode("utf-8")
            data = json.loads(body)
        except Exception:
            data = request.GET
            if not data:
                data = request.POST
            if not data:
                data = {}
    elif content_type == "application/json":
        return json.loads(request.body)
    elif content_type == "multipart/form-data":
        data = request.POST
    elif content_type == "application/xml":
        try:
            data = xmltodict.parse(request.body)
            return data.get("body") or data.get("data", {})
        except Exception as e:
            data = {}
    elif content_type == "application/x-www-form-urlencoded":
        data = parse_qs(request.body.decode())
        if data:
            data = {k: v[0] for k, v in data.items()}
        else:
            data = {}

    else:
        data = getattr(request, 'data', {})
    return {k: v for k, v in data.items()}


# 请求参数解析
def request_params_wrapper(func):
    # 解析请求参数 兼容 APIView与View的情况，View 没有request.data
    def wrapper(instance, arg_request=None, *args, request=None, **kwargs):
        """
        @param instance 实例是一个APIView的实例
        @param args 其它可变参数元组
        @param kwargs 其它可变关键字参数字典
        """
        if isinstance(instance, WSGIRequest) or isinstance(instance, Request) or isinstance(instance, ASGIRequest):
            request = instance
        if isinstance(arg_request, WSGIRequest) or isinstance(arg_request, Request) or isinstance(arg_request, ASGIRequest):
            request = arg_request
        if request is None:
            return func(instance, *args, request=request, request_params={}, **kwargs, )

        # 参数解析
        content_type = request.META.get('CONTENT_TYPE', "").split(";")[0]
        method = request.method
        # print("content_type:", content_type, "method:", method)
        if content_type == "text/plain" or method == "GET":
            try:
                body = request.body.decode("utf-8")
                data = json.loads(body)
            except Exception:
                data = request.GET
                if not data:
                    data = request.POST
                if not data:
                    data = {}
        elif content_type == "application/json":
            data = json.loads(request.body)
        elif content_type == "multipart/form-data":
            data = request.POST
        elif content_type == "application/xml":
            try:
                data = xmltodict.parse(request.body)
                data = data.get("body") or data.get("data", {})
            except Exception as e:
                data = {}
        elif content_type == "application/x-www-form-urlencoded":
            data = parse_qs(request.body.decode())
            if data:
                data = {k: v[0] for k, v in data.items()}
            else:
                data = {}
        else:
            data = getattr(request, 'data', {})
        # 闭包抛出
        return func(instance, *args, request=request, request_params={k: v for k, v in data.items()}, **kwargs, )

    return wrapper


def deal_equally(total: int, num: int):
    """
    发牌均分，发完截至, 然后打乱顺序
    :param total: 总数
    :param num: 平均分配给这些人
    :return: list
    """
    every_one_jetton = int((total / num))
    overplus_jetton = total % num
    jetton_list = [every_one_jetton for i in range(num)]
    if overplus_jetton == 0:
        return jetton_list
    for index in range(overplus_jetton):
        jetton_list[index] = every_one_jetton + 1
    random.shuffle(jetton_list)
    return jetton_list
